@@ -4,9 +4,12 @@ use quick_xml::{
     Writer,
 };
 use rustc_hash::FxHashMap;
-use std::io::{BufWriter, Cursor, Write};
-use std::path::Path;
 use std::time::{SystemTime, UNIX_EPOCH};
+use std::{
+    fmt::Display,
+    io::{BufWriter, Cursor, Write},
+};
+use std::{fmt::Formatter, path::Path};
 use symbolic_common::Name;
 use symbolic_demangle::{Demangle, DemangleOptions};
 
@@ -220,10 +223,10 @@ enum ConditionType {
     Jump,
 }
 
-impl ToString for ConditionType {
-    fn to_string(&self) -> String {
-        match *self {
-            Self::Jump => String::from("jump"),
+impl Display for ConditionType {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self {
+            Self::Jump => write!(f, "jump"),
         }
     }
 }
@@ -330,6 +333,7 @@ pub fn output_cobertura(
     results: &[ResultTuple],
     output_file: Option<&Path>,
     demangle: bool,
+    pretty: bool,
 ) {
     let demangle_options = DemangleOptions::name_only();
     let sources = vec![source_dir
@@ -338,7 +342,11 @@ pub fn output_cobertura(
         .to_string()];
     let coverage = get_coverage(results, sources, demangle, demangle_options);
 
-    let mut writer = Writer::new_with_indent(Cursor::new(vec![]), b' ', 4);
+    let mut writer = if pretty {
+        Writer::new_with_indent(Cursor::new(vec![]), b' ', 4)
+    } else {
+        Writer::new(Cursor::new(vec![]))
+    };
     writer
         .write_event(Event::Decl(BytesDecl::new("1.0", None, None)))
         .unwrap();
@@ -512,7 +520,7 @@ fn write_lines(writer: &mut Writer<Cursor<Vec<u8>>>, lines: &[Line]) {
             } => {
                 l.push_attribute(("number", number.to_string().as_ref()));
                 l.push_attribute(("hits", hits.to_string().as_ref()));
-                writer.write_event(Event::Start(l)).unwrap();
+                writer.write_event(Event::Empty(l)).unwrap();
             }
             Line::Branch {
                 ref number,
@@ -543,11 +551,11 @@ fn write_lines(writer: &mut Writer<Cursor<Vec<u8>>>, lines: &[Line]) {
                 writer
                     .write_event(Event::End(BytesEnd::new(conditions_tag)))
                     .unwrap();
+                writer
+                    .write_event(Event::End(BytesEnd::new(line_tag)))
+                    .unwrap();
             }
         }
-        writer
-            .write_event(Event::End(BytesEnd::new(line_tag)))
-            .unwrap();
     }
     writer
         .write_event(Event::End(BytesEnd::new(lines_tag)))
@@ -712,26 +720,28 @@ mod tests {
             coverage_result(Result::Main),
         )];
 
-        output_cobertura(None, &results, Some(&file_path), true);
+        for pretty in [false, true] {
+            output_cobertura(None, &results, Some(&file_path), true, pretty);
 
-        let results = read_file(&file_path);
+            let results = read_file(&file_path);
 
-        assert!(results.contains(r#"<source>.</source>"#));
+            assert!(results.contains(r#"<source>.</source>"#));
 
-        assert!(results.contains(r#"package name="src/main.rs""#));
-        assert!(results.contains(r#"class name="main" filename="src/main.rs""#));
-        assert!(results.contains(r#"method name="cov_test::main""#));
-        assert!(results.contains(r#"line number="1" hits="1">"#));
-        assert!(results.contains(r#"line number="3" hits="2" branch="true""#));
-        assert!(results.contains(r#"<condition number="0" type="jump" coverage="1"/>"#));
+            assert!(results.contains(r#"package name="src/main.rs""#));
+            assert!(results.contains(r#"class name="main" filename="src/main.rs""#));
+            assert!(results.contains(r#"method name="cov_test::main""#));
+            assert!(results.contains(r#"line number="1" hits="1"/>"#));
+            assert!(results.contains(r#"line number="3" hits="2" branch="true""#));
+            assert!(results.contains(r#"<condition number="0" type="jump" coverage="1"/>"#));
 
-        assert!(results.contains(r#"lines-covered="6""#));
-        assert!(results.contains(r#"lines-valid="8""#));
-        assert!(results.contains(r#"line-rate="0.75""#));
+            assert!(results.contains(r#"lines-covered="6""#));
+            assert!(results.contains(r#"lines-valid="8""#));
+            assert!(results.contains(r#"line-rate="0.75""#));
 
-        assert!(results.contains(r#"branches-covered="1""#));
-        assert!(results.contains(r#"branches-valid="4""#));
-        assert!(results.contains(r#"branch-rate="0.25""#));
+            assert!(results.contains(r#"branches-covered="1""#));
+            assert!(results.contains(r#"branches-valid="4""#));
+            assert!(results.contains(r#"branch-rate="0.25""#));
+        }
     }
 
     #[test]
@@ -746,7 +756,7 @@ mod tests {
             coverage_result(Result::Test),
         )];
 
-        output_cobertura(None, &results, Some(file_path.as_ref()), true);
+        output_cobertura(None, &results, Some(file_path.as_ref()), true, true);
 
         let results = read_file(&file_path);
 
@@ -785,7 +795,7 @@ mod tests {
             ),
         ];
 
-        output_cobertura(None, &results, Some(file_path.as_ref()), true);
+        output_cobertura(None, &results, Some(file_path.as_ref()), true, true);
 
         let results = read_file(&file_path);
 
@@ -817,7 +827,7 @@ mod tests {
             CovResult::default(),
         )];
 
-        output_cobertura(None, &results, Some(&file_path), true);
+        output_cobertura(None, &results, Some(&file_path), true, true);
 
         let results = read_file(&file_path);
 
@@ -837,7 +847,13 @@ mod tests {
             CovResult::default(),
         )];
 
-        output_cobertura(Some(Path::new("src")), &results, Some(&file_path), true);
+        output_cobertura(
+            Some(Path::new("src")),
+            &results,
+            Some(&file_path),
+            true,
+            true,
+        );
 
         let results = read_file(&file_path);
 
