@@ -1,7 +1,8 @@
-use cargo_binutils::Tool;
 use once_cell::sync::OnceCell;
 use rayon::prelude::{IntoParallelIterator, ParallelIterator};
+use std::env;
 use std::env::consts::EXE_SUFFIX;
+use std::error::Error;
 use std::ffi::OsStr;
 use std::fs;
 use std::io::Write;
@@ -146,12 +147,37 @@ pub fn profraws_to_lcov(
     Ok(results)
 }
 
+// The sysroot and rustlib functions are coming from https://github.com/rust-embedded/cargo-binutils/blob/a417523fa990c258509696507d1ce05f85dedbc4/src/rustc.rs.
+fn sysroot() -> Result<String, Box<dyn Error>> {
+    let rustc = env::var_os("RUSTC").unwrap_or_else(|| "rustc".into());
+    let output = Command::new(rustc).arg("--print").arg("sysroot").output()?;
+    // Note: We must trim() to remove the `\n` from the end of stdout
+    Ok(String::from_utf8(output.stdout)?.trim().to_owned())
+}
+
+// See: https://github.com/rust-lang/rust/blob/564758c4c329e89722454dd2fbb35f1ac0b8b47c/src/bootstrap/dist.rs#L2334-L2341
+fn rustlib() -> Result<PathBuf, Box<dyn Error>> {
+    let sysroot = sysroot()?;
+    let mut pathbuf = PathBuf::from(sysroot);
+    pathbuf.push("lib");
+    pathbuf.push("rustlib");
+    pathbuf.push(rustc_version::version_meta()?.host); // TODO: Prevent calling rustc_version::version_meta() multiple times
+    pathbuf.push("bin");
+    Ok(pathbuf)
+}
+
+fn llvm_tool_path(name: &str) -> Result<PathBuf, Box<dyn Error>> {
+    let mut path = rustlib()?;
+    path.push(format!("llvm-{}{}", name, EXE_SUFFIX));
+    Ok(path)
+}
+
 fn get_profdata_path() -> Result<PathBuf, String> {
     let path = if let Some(mut path) = LLVM_PATH.get().cloned() {
         path.push(format!("llvm-profdata{}", EXE_SUFFIX));
         path
     } else {
-        Tool::Profdata.path().map_err(|x| x.to_string())?
+        llvm_tool_path("profdata").map_err(|x| x.to_string())?
     };
 
     if !path.exists() {
@@ -166,7 +192,7 @@ fn get_cov_path() -> Result<PathBuf, String> {
         path.push(format!("llvm-cov{}", EXE_SUFFIX));
         path
     } else {
-        Tool::Cov.path().map_err(|x| x.to_string())?
+        llvm_tool_path("cov").map_err(|x| x.to_string())?
     };
 
     if !path.exists() {
@@ -245,21 +271,33 @@ mod tests {
         assert!(output_lcov
             .lines()
             .any(|line| line.contains("SF") && line.contains("src") && line.contains("main.rs")));
-        assert!(output_lcov.lines().any(|line| line.contains("FN:3")
-            && line.contains("rust_code_coverage_sample")
-            && line.contains("Ciao")));
+        if rustc_version::version_meta().unwrap().channel != rustc_version::Channel::Nightly {
+            assert!(output_lcov.lines().any(|line| line.contains("FN:3")
+                && line.contains("rust_code_coverage_sample")
+                && line.contains("Ciao")));
+        }
         assert!(output_lcov.lines().any(|line| line.contains("FN:8")
             && line.contains("rust_code_coverage_sample")
             && line.contains("main")));
-        assert!(output_lcov.lines().any(|line| line.contains("FNDA:0")
-            && line.contains("rust_code_coverage_sample")
-            && line.contains("Ciao")));
+        if rustc_version::version_meta().unwrap().channel != rustc_version::Channel::Nightly {
+            assert!(output_lcov.lines().any(|line| line.contains("FNDA:0")
+                && line.contains("rust_code_coverage_sample")
+                && line.contains("Ciao")));
+        } else {
+            assert!(output_lcov.lines().any(|line| line.contains("FNDA:1")
+                && line.contains("rust_code_coverage_sample")
+                && line.contains("main")));
+        }
         assert!(output_lcov.lines().any(|line| line.contains("FNDA:1")
             && line.contains("rust_code_coverage_sample")
             && line.contains("main")));
-        assert!(output_lcov.lines().any(|line| line == "FNF:2"));
+        if rustc_version::version_meta().unwrap().channel != rustc_version::Channel::Nightly {
+            assert!(output_lcov.lines().any(|line| line == "FNF:2"));
+        }
         assert!(output_lcov.lines().any(|line| line == "FNH:1"));
-        assert!(output_lcov.lines().any(|line| line == "DA:3,0"));
+        if rustc_version::version_meta().unwrap().channel != rustc_version::Channel::Nightly {
+            assert!(output_lcov.lines().any(|line| line == "DA:3,0"));
+        }
         assert!(output_lcov.lines().any(|line| line == "DA:8,1"));
         assert!(output_lcov.lines().any(|line| line == "DA:9,1"));
         assert!(output_lcov.lines().any(|line| line == "DA:10,1"));
@@ -267,8 +305,13 @@ mod tests {
         assert!(output_lcov.lines().any(|line| line == "DA:12,1"));
         assert!(output_lcov.lines().any(|line| line == "BRF:0"));
         assert!(output_lcov.lines().any(|line| line == "BRH:0"));
-        assert!(output_lcov.lines().any(|line| line == "LF:6"));
-        assert!(output_lcov.lines().any(|line| line == "LH:5"));
+        if rustc_version::version_meta().unwrap().channel == rustc_version::Channel::Nightly {
+            assert!(output_lcov.lines().any(|line| line == "LF:5"));
+            assert!(output_lcov.lines().any(|line| line == "LH:5"));
+        } else {
+            assert!(output_lcov.lines().any(|line| line == "LF:6"));
+            assert!(output_lcov.lines().any(|line| line == "LH:5"));
+        }
         assert!(output_lcov.lines().any(|line| line == "end_of_record"));
     }
 }
