@@ -19,7 +19,7 @@ use std::{process, thread};
 
 use grcov::*;
 
-#[derive(Clone)]
+#[derive(Clone, PartialEq)]
 enum OutputType {
     Ade,
     Lcov,
@@ -29,6 +29,7 @@ enum OutputType {
     Covdir,
     Html,
     Cobertura,
+    CoberturaPretty,
     Markdown,
 }
 
@@ -45,6 +46,7 @@ impl FromStr for OutputType {
             "covdir" => Self::Covdir,
             "html" => Self::Html,
             "cobertura" => Self::Cobertura,
+            "cobertura-pretty" => Self::CoberturaPretty,
             "markdown" => Self::Markdown,
             _ => return Err(format!("{} is not a supported output type", s)),
         })
@@ -63,7 +65,9 @@ impl OutputType {
                     OutputType::Files => path.join("files"),
                     OutputType::Covdir => path.join("covdir"),
                     OutputType::Html => path.join("html"),
-                    OutputType::Cobertura => path.join("cobertura.xml"),
+                    OutputType::Cobertura | OutputType::CoberturaPretty => {
+                        path.join("cobertura.xml")
+                    }
                     OutputType::Markdown => path.join("markdown.md"),
                 }
             } else {
@@ -166,6 +170,7 @@ struct Opt {
             - *files* to only return a list of files.\n\
             - *markdown* for human easy read.\n\
             - *cobertura* for output in cobertura format.\n\
+            - *cobertura-pretty* to pretty-print in cobertura format.\n\
         ",
         value_name = "OUTPUT TYPE",
         requires_ifs = [
@@ -210,6 +215,15 @@ struct Opt {
     /// to only return uncovered files.
     #[arg(long, value_enum)]
     filter: Option<Filter>,
+    /// Comma separated list of output types to sort files lexicographically for.
+    #[arg(
+        long,
+        value_name = "OUTPUT TYPES",
+        value_delimiter = ',',
+        alias = "sort-output-types",
+        default_value = "markdown"
+    )]
+    sort_output_types: Vec<OutputType>,
     /// Speeds-up parsing, when the code coverage information is exclusively coming from a llvm
     /// build.
     #[arg(long)]
@@ -469,6 +483,7 @@ fn main() {
         filter_option,
         file_filter,
     );
+    let mut sorted_iterator: Option<Vec<ResultTuple>> = None;
 
     let service_number = opt.service_number.unwrap_or_default();
     let service_pull_request = opt.service_pull_request.unwrap_or_default();
@@ -493,12 +508,23 @@ fn main() {
 
     for output_type in &output_types {
         let output_path = output_type.to_file_name(output_path);
+        let results = if opt.sort_output_types.contains(output_type) {
+            // compute and cache the sorted results if not already used
+            sorted_iterator = sorted_iterator.or_else(|| {
+                let mut results = iterator.clone();
+                results.sort_by_key(|result| result.0.display().to_string());
+                Some(results)
+            });
+            sorted_iterator.as_ref().unwrap()
+        } else {
+            &iterator
+        };
 
         match output_type {
-            OutputType::Ade => output_activedata_etl(&iterator, output_path.as_deref(), demangle),
-            OutputType::Lcov => output_lcov(&iterator, output_path.as_deref(), demangle),
+            OutputType::Ade => output_activedata_etl(results, output_path.as_deref(), demangle),
+            OutputType::Lcov => output_lcov(results, output_path.as_deref(), demangle),
             OutputType::Coveralls => output_coveralls(
-                &iterator,
+                results,
                 opt.token.as_deref(),
                 opt.service_name.as_deref(),
                 &service_number,
@@ -513,7 +539,7 @@ fn main() {
                 demangle,
             ),
             OutputType::CoverallsPlus => output_coveralls(
-                &iterator,
+                results,
                 opt.token.as_deref(),
                 opt.service_name.as_deref(),
                 &service_number,
@@ -527,10 +553,10 @@ fn main() {
                 opt.parallel,
                 demangle,
             ),
-            OutputType::Files => output_files(&iterator, output_path.as_deref()),
-            OutputType::Covdir => output_covdir(&iterator, output_path.as_deref(), opt.precision),
+            OutputType::Files => output_files(results, output_path.as_deref()),
+            OutputType::Covdir => output_covdir(results, output_path.as_deref(), opt.precision),
             OutputType::Html => output_html(
-                &iterator,
+                results,
                 output_path.as_deref(),
                 num_threads,
                 opt.branch,
@@ -539,13 +565,19 @@ fn main() {
             ),
             OutputType::Cobertura => output_cobertura(
                 source_root.as_deref(),
-                &iterator,
+                results,
                 output_path.as_deref(),
                 demangle,
+                false,
             ),
-            OutputType::Markdown => {
-                output_markdown(&iterator, output_path.as_deref(), opt.precision)
-            }
+            OutputType::CoberturaPretty => output_cobertura(
+                source_root.as_deref(),
+                results,
+                output_path.as_deref(),
+                demangle,
+                true,
+            ),
+            OutputType::Markdown => output_markdown(results, output_path.as_deref(), opt.precision),
         };
     }
 }
