@@ -3,7 +3,7 @@ use md5::{Digest, Md5};
 use rustc_hash::FxHashMap;
 use serde_json::{self, json, Value};
 use std::cell::RefCell;
-use std::collections::{hash_map, BTreeSet};
+use std::collections::{hash_map, BTreeMap, BTreeSet};
 use std::ffi::OsStr;
 use std::fs::File;
 use std::io::{self, BufWriter, Read, Write};
@@ -16,6 +16,8 @@ use std::{
 };
 use symbolic_common::Name;
 use symbolic_demangle::{Demangle, DemangleOptions};
+use tabled::settings::Style;
+use tabled::{Table, Tabled};
 use uuid::Uuid;
 
 use crate::defs::*;
@@ -44,12 +46,12 @@ pub fn get_target_output_writable(output_file: Option<&Path>) -> Box<dyn Write>
                     output.display()
                 )
             }
-            Box::new(File::create(&output).unwrap_or_else(|_| {
+            Box::new(File::create(output).unwrap_or_else(|_| {
                 let parent = output.parent();
                 if let Some(parent_path) = parent {
                     if !parent_path.exists() {
                         panic!(
-                            "Cannot create {} to dump coverage data, as {} doesn't exist",
+                            "Cannot create file {} to dump coverage data, as the parent directory {} doesn't exist.",
                             output.display(),
                             parent_path.display()
                         )
@@ -69,7 +71,7 @@ pub fn get_target_output_writable(output_file: Option<&Path>) -> Box<dyn Write>
     write_target
 }
 
-pub fn output_activedata_etl(results: CovResultIter, output_file: Option<&Path>, demangle: bool) {
+pub fn output_activedata_etl(results: &[ResultTuple], output_file: Option<&Path>, demangle: bool) {
     let demangle_options = DemangleOptions::name_only();
     let mut writer = BufWriter::new(get_target_output_writable(output_file));
 
@@ -179,7 +181,7 @@ pub fn output_activedata_etl(results: CovResultIter, output_file: Option<&Path>,
     }
 }
 
-pub fn output_covdir(results: CovResultIter, output_file: Option<&Path>) {
+pub fn output_covdir(results: &[ResultTuple], output_file: Option<&Path>, precision: usize) {
     let mut writer = BufWriter::new(get_target_output_writable(output_file));
     let mut relative: FxHashMap<PathBuf, Rc<RefCell<CDDirStats>>> = FxHashMap::default();
     let global = Rc::new(RefCell::new(CDDirStats::new("".to_string())));
@@ -225,17 +227,18 @@ pub fn output_covdir(results: CovResultIter, output_file: Option<&Path>) {
 
         prev_stats.borrow_mut().files.push(CDFileStats::new(
             path.file_name().unwrap().to_str().unwrap().to_string(),
-            result.lines,
+            result.lines.clone(),
+            precision,
         ));
     }
 
     let mut global = global.take();
-    global.set_stats();
+    global.set_stats(precision);
 
     serde_json::to_writer(&mut writer, &global.into_json()).unwrap();
 }
 
-pub fn output_lcov(results: CovResultIter, output_file: Option<&Path>, demangle: bool) {
+pub fn output_lcov(results: &[ResultTuple], output_file: Option<&Path>, demangle: bool) {
     let demangle_options = DemangleOptions::name_only();
     let mut writer = BufWriter::new(get_target_output_writable(output_file));
     writer.write_all(b"TN:\n").unwrap();
@@ -258,7 +261,7 @@ pub fn output_lcov(results: CovResultIter, output_file: Option<&Path>, demangle:
             writeln!(
                 writer,
                 "FNDA:{},{}",
-                if function.executed { 1 } else { 0 },
+                i32::from(function.executed),
                 demangle!(name, demangle, demangle_options)
             )
             .unwrap();
@@ -368,7 +371,7 @@ fn get_coveralls_git_info(commit_sha: &str, vcs_branch: &str) -> Value {
     // Runs `git log` with a given format, to extract some piece of commit info. On failure,
     // returns empty string.
     let gitlog = |format| -> String {
-        get_git_output(&[
+        get_git_output([
             "log",
             "--max-count=1",
             &format!("--pretty=format:{}", format),
@@ -383,7 +386,7 @@ fn get_coveralls_git_info(commit_sha: &str, vcs_branch: &str) -> Value {
     let message = gitlog("%s");
 
     let remotes: Value = {
-        let output = get_git_output(&["remote", "--verbose"]);
+        let output = get_git_output(["remote", "--verbose"]);
 
         let mut remotes = Vec::<Value>::new();
         for line in output.lines() {
@@ -412,12 +415,13 @@ fn get_coveralls_git_info(commit_sha: &str, vcs_branch: &str) -> Value {
 }
 
 pub fn output_coveralls(
-    results: CovResultIter,
+    results: &[ResultTuple],
     repo_token: Option<&str>,
     service_name: Option<&str>,
     service_number: &str,
     service_job_id: Option<&str>,
     service_pull_request: &str,
+    service_flag_name: Option<&str>,
     commit_sha: &str,
     with_function_info: bool,
     output_file: Option<&Path>,
@@ -447,14 +451,14 @@ pub fn output_coveralls(
                 branches.push(*line);
                 branches.push(0);
                 branches.push(n as u32);
-                branches.push(if *b_t { 1 } else { 0 });
+                branches.push(u32::from(*b_t));
             }
         }
 
         if !with_function_info {
             source_files.push(json!({
                 "name": rel_path,
-                "source_digest": get_digest(abs_path),
+                "source_digest": get_digest(abs_path.clone()),
                 "coverage": coverage,
                 "branches": branches,
             }));
@@ -470,7 +474,7 @@ pub fn output_coveralls(
 
             source_files.push(json!({
                 "name": rel_path,
-                "source_digest": get_digest(abs_path),
+                "source_digest": get_digest(abs_path.clone()),
                 "coverage": coverage,
                 "branches": branches,
                 "functions": functions,
@@ -496,6 +500,10 @@ pub fn output_coveralls(
         obj.insert("service_name".to_string(), json!(service_name));
     }
 
+    if let (Some(service_flag_name), Some(obj)) = (service_flag_name, result.as_object_mut()) {
+        obj.insert("flag_name".to_string(), json!(service_flag_name));
+    }
+
     if let (Some(service_job_id), Some(obj)) = (service_job_id, result.as_object_mut()) {
         obj.insert("service_job_id".to_string(), json!(service_job_id));
     }
@@ -504,7 +512,7 @@ pub fn output_coveralls(
     serde_json::to_writer(&mut writer, &result).unwrap();
 }
 
-pub fn output_files(results: CovResultIter, output_file: Option<&Path>) {
+pub fn output_files(results: &[ResultTuple], output_file: Option<&Path>) {
     let mut writer = BufWriter::new(get_target_output_writable(output_file));
     for (_, rel_path, _) in results {
         writeln!(writer, "{}", rel_path.display()).unwrap();
@@ -512,11 +520,12 @@ pub fn output_files(results: CovResultIter, output_file: Option<&Path>) {
 }
 
 pub fn output_html(
-    results: CovResultIter,
+    results: &[ResultTuple],
     output_dir: Option<&Path>,
     num_threads: usize,
     branch_enabled: bool,
     output_config_file: Option<&Path>,
+    precision: usize,
 ) {
     let output = if let Some(output_dir) = output_dir {
         PathBuf::from(output_dir)
@@ -529,7 +538,7 @@ pub fn output_html(
             eprintln!("{} is not a directory", output.to_str().unwrap());
             return;
         }
-    } else if std::fs::create_dir(&output).is_err() {
+    } else if std::fs::create_dir_all(&output).is_err() {
         eprintln!("Cannot create directory {}", output.to_str().unwrap());
         return;
     }
@@ -548,7 +557,15 @@ pub fn output_html(
         let t = thread::Builder::new()
             .name(format!("Consumer HTML {}", i))
             .spawn(move || {
-                html::consumer_html(&tera, receiver, stats, &output, config, branch_enabled);
+                html::consumer_html(
+                    &tera,
+                    receiver,
+                    stats,
+                    &output,
+                    config,
+                    branch_enabled,
+                    precision,
+                );
             })
             .unwrap();
 
@@ -558,9 +575,9 @@ pub fn output_html(
     for (abs_path, rel_path, result) in results {
         sender
             .send(Some(HtmlItem {
-                abs_path,
-                rel_path,
-                result,
+                abs_path: abs_path.to_path_buf(),
+                rel_path: rel_path.to_path_buf(),
+                result: result.clone(),
             }))
             .unwrap();
     }
@@ -577,13 +594,82 @@ pub fn output_html(
 
     let global = Arc::try_unwrap(stats).unwrap().into_inner().unwrap();
 
-    html::gen_index(&tera, &global, &config, &output, branch_enabled);
+    html::gen_index(&tera, &global, &config, &output, branch_enabled, precision);
 
     for style in html::BadgeStyle::iter() {
         html::gen_badge(&tera, &global.stats, &config, &output, style);
     }
 
-    html::gen_coverage_json(&global.stats, &config, &output);
+    html::gen_coverage_json(&global.stats, &config, &output, precision);
+}
+
+pub fn output_markdown(results: &[ResultTuple], output_file: Option<&Path>, precision: usize) {
+    #[derive(Tabled)]
+    struct LineSummary {
+        file: String,
+        coverage: String,
+        covered: String,
+        missed_lines: String,
+    }
+
+    fn format_pair(start: u32, end: u32) -> String {
+        if start == end {
+            start.to_string()
+        } else {
+            format!("{}-{}", start, end)
+        }
+    }
+
+    fn format_lines(lines: &BTreeMap<u32, u64>) -> (usize, String) {
+        let mut total_missed = 0;
+        let mut missed = Vec::new();
+        let mut start: u32 = 0;
+        let mut end: u32 = 0;
+        for (&line, &hits) in lines {
+            if hits == 0 {
+                total_missed += 1;
+                if start == 0 {
+                    start = line;
+                }
+                end = line;
+            } else if start != 0 {
+                missed.push(format_pair(start, end));
+                start = 0;
+            }
+        }
+        if start != 0 {
+            missed.push(format_pair(start, end));
+        }
+        (total_missed, missed.join(", "))
+    }
+
+    let mut summary = Vec::new();
+    let mut total_lines: usize = 0;
+    let mut total_covered: usize = 0;
+    for (_, rel_path, result) in results {
+        let (missed, missed_lines) = format_lines(&result.lines);
+        let covered: usize = result.lines.len() - missed;
+        summary.push(LineSummary {
+            file: rel_path.display().to_string(),
+            coverage: format!(
+                "{:.precision$}%",
+                (covered as f32 * 100.0 / result.lines.len() as f32),
+            ),
+            covered: format!("{} / {}", covered, result.lines.len()),
+            missed_lines,
+        });
+        total_lines += result.lines.len();
+        total_covered += covered;
+    }
+    let mut writer = BufWriter::new(get_target_output_writable(output_file));
+    writeln!(writer, "{}", Table::new(summary).with(Style::markdown())).unwrap();
+    writeln!(writer).unwrap();
+    writeln!(
+        writer,
+        "Total coverage: {:.precision$}%",
+        (total_covered as f32 * 100.0 / total_lines as f32),
+    )
+    .unwrap()
 }
 
 #[cfg(test)]
@@ -603,7 +689,7 @@ mod tests {
     fn test_lcov_brf_brh() {
         let tmp_dir = tempfile::tempdir().expect("Failed to create temporary directory");
         let file_name = "test_lcov_brf_brh.info";
-        let file_path = tmp_dir.path().join(&file_name);
+        let file_path = tmp_dir.path().join(file_name);
 
         let results = vec![(
             PathBuf::from("foo/bar/a.cpp"),
@@ -621,8 +707,7 @@ mod tests {
             },
         )];
 
-        let results = Box::new(results.into_iter());
-        output_lcov(results, Some(&file_path), false);
+        output_lcov(&results, Some(&file_path), false);
 
         let results = read_file(&file_path);
 
@@ -634,7 +719,7 @@ mod tests {
     fn test_lcov_demangle() {
         let tmp_dir = tempfile::tempdir().expect("Failed to create temporary directory");
         let file_name = "test_lcov_demangle";
-        let file_path = tmp_dir.path().join(&file_name);
+        let file_path = tmp_dir.path().join(file_name);
 
         let results = vec![(
             PathBuf::from("foo/bar/a.cpp"),
@@ -670,8 +755,7 @@ mod tests {
             },
         )];
 
-        let results = Box::new(results.into_iter());
-        output_lcov(results, Some(&file_path), true);
+        output_lcov(&results, Some(&file_path), true);
 
         let results = read_file(&file_path);
 
@@ -684,7 +768,7 @@ mod tests {
     fn test_covdir() {
         let tmp_dir = tempfile::tempdir().expect("Failed to create temporary directory");
         let file_name = "test_covdir.json";
-        let file_path = tmp_dir.path().join(&file_name);
+        let file_path = tmp_dir.path().join(file_name);
 
         let results = vec![
             (
@@ -725,11 +809,10 @@ mod tests {
             ),
         ];
 
-        let results = Box::new(results.into_iter());
-        output_covdir(results, Some(&file_path));
+        output_covdir(&results, Some(&file_path), 2);
 
         let results: Value = serde_json::from_str(&read_file(&file_path)).unwrap();
-        let expected_path = PathBuf::from("./test/").join(&file_name);
+        let expected_path = PathBuf::from("./test/").join(file_name);
         let expected: Value = serde_json::from_str(&read_file(&expected_path)).unwrap();
 
         assert_eq!(results, expected);
@@ -739,7 +822,7 @@ mod tests {
     fn test_coveralls_service_job_id() {
         let tmp_dir = tempfile::tempdir().expect("Failed to create temporary directory");
         let file_name = "test_coveralls_service_job_id.json";
-        let file_path = tmp_dir.path().join(&file_name);
+        let file_path = tmp_dir.path().join(file_name);
 
         let results = vec![(
             PathBuf::from("foo/bar/a.cpp"),
@@ -751,17 +834,17 @@ mod tests {
             },
         )];
 
-        let results = Box::new(results.into_iter());
         let expected_service_job_id: &str = "100500";
         let with_function_info: bool = true;
         let parallel: bool = true;
         output_coveralls(
-            results,
+            &results,
             None,
             None,
             "unused",
             Some(expected_service_job_id),
             "unused",
+            Some("unused"),
             "unused",
             with_function_info,
             Some(&file_path),
@@ -776,10 +859,51 @@ mod tests {
     }
 
     #[test]
-    fn test_coveralls_token_field_is_absent_if_arg_is_none() {
+    fn test_coveralls_service_flag_name() {
         let tmp_dir = tempfile::tempdir().expect("Failed to create temporary directory");
+        let file_name = "test_coveralls_service_job_id.json";
+        let file_path = tmp_dir.path().join(file_name);
+
+        let results = vec![(
+            PathBuf::from("foo/bar/a.cpp"),
+            PathBuf::from("foo/bar/a.cpp"),
+            CovResult {
+                lines: [(1, 10), (2, 11)].iter().cloned().collect(),
+                branches: BTreeMap::new(),
+                functions: FxHashMap::default(),
+            },
+        )];
+
+        let expected_service_job_id: &str = "100500";
+        let expected_flag_name: &str = "expected flag name";
+        let with_function_info: bool = true;
+        let parallel: bool = true;
+        output_coveralls(
+            &results,
+            None,
+            None,
+            "unused",
+            Some(expected_service_job_id),
+            "unused",
+            Some(expected_flag_name),
+            "unused",
+            with_function_info,
+            Some(&file_path),
+            "unused",
+            parallel,
+            false,
+        );
+
+        let results: Value = serde_json::from_str(&read_file(&file_path)).unwrap();
+
+        assert_eq!(results["flag_name"], expected_flag_name);
+    }
+
+    #[test]
+    fn test_coveralls_token_field_is_absent_if_arg_is_none() {
+        let tmp_dir = tempfile::tempdir().expect("Failed to create temporary directory");
         let file_name = "test_coveralls_token.json";
-        let file_path = tmp_dir.path().join(&file_name);
+        let file_path = tmp_dir.path().join(file_name);
 
         let results = vec![(
             PathBuf::from("foo/bar/a.cpp"),
@@ -791,17 +915,17 @@ mod tests {
             },
         )];
 
-        let results = Box::new(results.into_iter());
         let token = None;
         let with_function_info: bool = true;
         let parallel: bool = true;
         output_coveralls(
-            results,
+            &results,
             token,
             None,
             "unused",
             None,
             "unused",
+            Some("unused"),
             "unused",
             with_function_info,
             Some(&file_path),
@@ -819,7 +943,7 @@ mod tests {
     fn test_coveralls_service_fields_are_absent_if_args_are_none() {
         let tmp_dir = tempfile::tempdir().expect("Failed to create temporary directory");
         let file_name = "test_coveralls_service_fields.json";
-        let file_path = tmp_dir.path().join(&file_name);
+        let file_path = tmp_dir.path().join(file_name);
 
         let results = vec![(
             PathBuf::from("foo/bar/a.cpp"),
@@ -831,18 +955,18 @@ mod tests {
             },
         )];
 
-        let results = Box::new(results.into_iter());
         let service_name = None;
         let service_job_id = None;
         let with_function_info: bool = true;
         let parallel: bool = true;
         output_coveralls(
-            results,
+            &results,
             None,
             service_name,
             "unused",
             service_job_id,
             "unused",
+            None,
             "unused",
             with_function_info,
             Some(&file_path),
@@ -855,5 +979,49 @@ mod tests {
 
         assert_eq!(results.get("service_name"), None);
         assert_eq!(results.get("service_job_id"), None);
+        assert_eq!(results.get("flag_name"), None)
+    }
+
+    #[test]
+    fn test_markdown() {
+        let tmp_dir = tempfile::tempdir().expect("Failed to create temporary directory");
+        let file_name = "test_markdown";
+        let file_path = tmp_dir.path().join(file_name);
+
+        let results = vec![
+            (
+                PathBuf::from("foo/bar/a.cpp"),
+                PathBuf::from("foo/bar/a.cpp"),
+                CovResult {
+                    lines: [(1, 10), (2, 11)].iter().cloned().collect(),
+                    branches: BTreeMap::new(),
+                    functions: FxHashMap::default(),
+                },
+            ),
+            (
+                PathBuf::from("foo/bar/b.cpp"),
+                PathBuf::from("foo/bar/b.cpp"),
+                CovResult {
+                    lines: [(1, 0), (2, 10), (4, 10), (5, 0), (7, 0)]
+                        .iter()
+                        .cloned()
+                        .collect(),
+                    branches: BTreeMap::new(),
+                    functions: FxHashMap::default(),
+                },
+            ),
+        ];
+
+        output_markdown(&results, Some(&file_path), 2);
+
+        let results = &read_file(&file_path);
+        let expected = "| file          | coverage | covered | missed_lines |
+|---------------|----------|---------|--------------|
+| foo/bar/a.cpp | 100.00%  | 2 / 2   |              |
+| foo/bar/b.cpp | 40.00%   | 2 / 5   | 1, 5-7       |
+
+Total coverage: 57.14%
+";
+        assert_eq!(results, expected);
     }
 }
